/*Asic_usb.c: function of IO;*/
/* sane - Scanner Access Now Easy.

   Copyright (C) 2005 Mustek.
   Originally maintained by Mustek
   Author:Roy 2005.5.24

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB and similar 
   USB flatbed scanners.  */
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <dirent.h>
#include <linux/usb.h>

#include "asic_usb.h"
#include "asic_libusb.h"

#ifndef HAVE_LIBUSB
#define HAVE_LIBUSB
#endif

BYTE RegisterBankStatus = -1;


STATUS WriteIOControl(PAsic chip, WORD wValue, WORD wIndex, WORD wLength, LPBYTE lpbuf)
{
	STATUS status = STATUS_GOOD;
	
	status = libusb_control_msg (chip->fd, 0x40,0x01,wValue, wIndex, wLength,lpbuf);
	if (status != STATUS_GOOD)
    {
		XDBG(DEBUG_ASIC,"WriteIOControl Error!");
		return status;
    }
	
	return STATUS_GOOD;
}

STATUS ReadIOControl(PAsic chip, WORD wValue, WORD wIndex, WORD wLength, LPBYTE lpbuf)
{
	STATUS status = STATUS_GOOD;
	
	status = libusb_control_msg (chip->fd, 0xc0, 0x01,wValue, wIndex, wLength,lpbuf);
	if (status != STATUS_GOOD)
    {
		XDBG(DEBUG_ASIC,"WriteIOControl Error!");
		return status;
    }
	
	return status;
}

STATUS Mustek_ClearFIFO(PAsic chip)
{
	STATUS status= STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Mustek_ClearFIFO:Enter");
	
	BYTE buf[4];
	buf[0]=0;
	buf[1]=0;
	buf[2]=0;
	buf[3]=0;
	status = WriteIOControl(chip,0x05,0,4,(LPBYTE)(buf)); 
	if (status != STATUS_GOOD)
		return status;
	
	status = WriteIOControl(chip,0xc0,0,4,(LPBYTE)(buf)); 
	if (status != STATUS_GOOD)
		return status;
	
	
	XDBG(DEBUG_ASIC,"Mustek_ClearFIFO:Exit");
	return STATUS_GOOD;
}


STATUS Mustek_SendData(PAsic chip, WORD reg, BYTE data)
{
	BYTE buf[4];
	STATUS status= STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Mustek_SendData: Enter. reg=%x,data=%x",reg,data);
	
	
	if (reg <= 0xFF)
	{
		if (RegisterBankStatus != 0)
		{
			XDBG(DEBUG_ASIC,"RegisterBankStatus=%d",RegisterBankStatus);
			buf[0]=ES01_5F_REGISTER_BANK_SELECT;
			buf[1]=SELECT_REGISTER_BANK0;
			buf[2]=ES01_5F_REGISTER_BANK_SELECT;
			buf[3]=SELECT_REGISTER_BANK0;
			WriteIOControl(chip,0xb0,0,4,buf);
			RegisterBankStatus = 0;
			XDBG(DEBUG_ASIC,"RegisterBankStatus=%d",RegisterBankStatus);  
		}
		
	}
	else if (reg <= 0x1FF)
	{
		if (RegisterBankStatus != 1)
		{
			XDBG(DEBUG_ASIC,"RegisterBankStatus=%d",RegisterBankStatus);
			buf[0]=ES01_5F_REGISTER_BANK_SELECT;
			buf[1]=SELECT_REGISTER_BANK1;
			buf[2]=ES01_5F_REGISTER_BANK_SELECT;
			buf[3]=SELECT_REGISTER_BANK1;

			WriteIOControl(chip,0xb0,0,4,buf);
			RegisterBankStatus = 1;
		}
	}
	else if (reg <= 0x2FF)
	{
		if (RegisterBankStatus != 2)
		{
			XDBG(DEBUG_ASIC,"RegisterBankStatus=%d",RegisterBankStatus);
			buf[0]=ES01_5F_REGISTER_BANK_SELECT;
			buf[1]=SELECT_REGISTER_BANK2;
			buf[2]=ES01_5F_REGISTER_BANK_SELECT;
			buf[3]=SELECT_REGISTER_BANK2;
			
			WriteIOControl(chip,0xb0,0,4,buf);
			RegisterBankStatus = 2;
		}
	}
	
	buf[0] = LOBYTE(reg); 
	buf[1] = data;
	buf[2] = LOBYTE(reg); 
	buf[3] = data;
	status = WriteIOControl(chip,0xb0,0,4,buf);
	if (status != STATUS_GOOD)
		perror("Mustek_SendData: write error\n");
	
	return status;
}

STATUS Mustek_ReceiveData(PAsic chip, LPBYTE reg)
{
	STATUS status= STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Mustek_ReceiveData");
	BYTE buf[4];
	
	status = ReadIOControl(chip, 0x07, 0, 4, buf);
	
	*reg = buf[0];
	return status;
}

STATUS Mustek_WriteAddressLineForRegister(PAsic chip, BYTE x)
{
	STATUS status= STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Mustek_WriteAddressLineForRegister: Enter");
	
	BYTE buf[4];
	buf[0] = x;
	buf[1] = x;
	buf[2] = x;
	buf[3] = x;
	status = WriteIOControl(chip,0x04,x,4,buf);
	
	XDBG(DEBUG_ASIC,"Mustek_WriteAddressLineForRegister: Exit");
	return status;
}


STATUS SetRWSize(PAsic chip,BYTE ReadWrite, DWORD size)
{
	STATUS status= STATUS_GOOD;
	XDBG(DEBUG_ASIC,"SetRWSize: Enter");
	
	if (ReadWrite == 0)
	{/*write */
		status =Mustek_SendData(chip,0x7C, (BYTE)(size));
		if (status != STATUS_GOOD) return status;
		status =Mustek_SendData(chip,0x7D, (BYTE)(size>>8));
		if (status != STATUS_GOOD) return status;
		status =Mustek_SendData(chip,0x7E, (BYTE)(size>>16));
		if (status != STATUS_GOOD) return status;
		status =Mustek_SendData(chip,0x7F, (BYTE)(size>>24));
		if (status != STATUS_GOOD) return status;
	}
	else
	{/* read */
		status =Mustek_SendData(chip,0x7C, (BYTE)(size>>1));
		if (status != STATUS_GOOD) return status;
		status =Mustek_SendData(chip,0x7D, (BYTE)(size>>9));
		if (status != STATUS_GOOD) return status;
		status =Mustek_SendData(chip,0x7E, (BYTE)(size>>17));
		if (status != STATUS_GOOD) return status;
		status =Mustek_SendData(chip,0x7F, (BYTE)(size>>25));
		if (status != STATUS_GOOD) return status;
	}	
	
	XDBG(DEBUG_ASIC,"SetRWSize: Exit");
	return STATUS_GOOD;
}

STATUS Mustek_DMARead(PAsic chip, DWORD size,LPBYTE lpdata)
{
	STATUS status= STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Mustek_DMARead: Enter");
	DWORD i,buf[1],rate;
	DWORD read_size;
	
	status =Mustek_ClearFIFO(chip);
	if (status != STATUS_GOOD)
		return status;
	
	buf[0] = read_size =  32*1024;
	for(i=0;i<size/(read_size);i++)
	{
		SetRWSize(chip,1, buf[0]); 
		status = WriteIOControl(chip,0x03,0,4,(LPBYTE)(buf));

		status =libusb_read_bulk (chip->fd, lpdata+i*read_size, (size_t*)buf);
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"Mustek_DMARead: read error");
			return status;
		}
	}
	
	buf[0] = size - i*read_size;
	if(buf[0] > 0)
	{
		SetRWSize (chip,1, buf[0]);
		status = WriteIOControl(chip,0x03,0,4,(LPBYTE)(buf));

		status =libusb_read_bulk (chip->fd, lpdata+i*read_size, (size_t*)buf);
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"Mustek_DMARead: read error");
			return status;
		}
	
		usleep(20000);
	}
	
	XDBG(DEBUG_ASIC,"Mustek_DMARead: Exit");
	return STATUS_GOOD;
}

STATUS Mustek_DMAWrite(PAsic chip,DWORD size, LPBYTE lpdata)
{
	STATUS status= STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Mustek_DMAWrite: Enter:size=%d",size);
	DWORD buf[1];
	DWORD i;
	DWORD write_size;
	
	status =Mustek_ClearFIFO(chip);
	if (status != STATUS_GOOD)
		return status;
	
	buf[0] =write_size = 32*1024;
	for(i=0;i<size/(write_size);i++)
	{
		SetRWSize (chip,0, buf[0]);
		WriteIOControl(chip,0x02,0,4,(LPBYTE)buf);

		status =libusb_write_bulk (chip->fd, lpdata+i*write_size, (size_t*)buf);
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"Mustek_DMAWrite: write error");
			return status;
		}
	}
	
	
	buf[0] = size - i*write_size;
	if(buf[0] > 0)
	{
		SetRWSize (chip,0, buf[0]);
		WriteIOControl(chip,0x02,0,4,(LPBYTE)buf);

		status =libusb_write_bulk (chip->fd, lpdata+i*write_size, (size_t*)buf);
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"Mustek_DMAWrite: write error");
			return status;
		}
	}
	
	Mustek_ClearFIFO(chip);
	
	XDBG(DEBUG_ASIC,"Mustek_DMAWrite: Exit");
	return STATUS_GOOD;
}


STATUS Mustek_SendData2Byte(PAsic chip, WORD reg, BYTE data)
{
	STATUS status= STATUS_GOOD;
	static BOOL isTransfer=FALSE;
	static BYTE BankBuf[4];
	static BYTE DataBuf[4];
	
	if (reg <= 0xFF)
	{
		if (RegisterBankStatus != 0)
		{
			XDBG(DEBUG_ASIC,"RegisterBankStatus=%d",RegisterBankStatus);
			BankBuf[0]=ES01_5F_REGISTER_BANK_SELECT;
			BankBuf[1]=SELECT_REGISTER_BANK0;
			BankBuf[2]=ES01_5F_REGISTER_BANK_SELECT;
			BankBuf[3]=SELECT_REGISTER_BANK0;
			WriteIOControl(chip,0xb0,0,4,BankBuf);
			
			RegisterBankStatus = 0;
		}
	}
	else if (reg <= 0x1FF)
	{
		if (RegisterBankStatus != 1)
		{
			XDBG(DEBUG_ASIC,"RegisterBankStatus=%d",RegisterBankStatus);
			BankBuf[0]=ES01_5F_REGISTER_BANK_SELECT;
			BankBuf[1]=SELECT_REGISTER_BANK1;
			BankBuf[2]=ES01_5F_REGISTER_BANK_SELECT;
			
			BankBuf[3]=SELECT_REGISTER_BANK1;
			WriteIOControl(chip,0xb0,0,4,BankBuf);
			RegisterBankStatus = 1;
		}
	}
	else if (reg <= 0x2FF)
	{
		if (RegisterBankStatus != 2)
		{
			XDBG(DEBUG_ASIC,"RegisterBankStatus=%d",RegisterBankStatus);
			BankBuf[0]=ES01_5F_REGISTER_BANK_SELECT;
			BankBuf[1]=SELECT_REGISTER_BANK2;
			BankBuf[2]=ES01_5F_REGISTER_BANK_SELECT;
			BankBuf[3]=SELECT_REGISTER_BANK2;
			WriteIOControl(chip,0xb0,0,4,BankBuf);
			RegisterBankStatus = 2;
		}
	}
	
	if(isTransfer == FALSE)
	{
		DataBuf[0] = LOBYTE(reg); 
		DataBuf[1] = data;
		isTransfer = TRUE;
	}
	else
	{
		DataBuf[2] = LOBYTE(reg); 
		DataBuf[3] = data;
		WriteIOControl(chip,0xb0,0,4,DataBuf);
		isTransfer = FALSE;
	}
	
	return STATUS_GOOD;
}
