/* sane - Scanner Access Now Easy.

   Copyright (C) 2005 Mustek.
   Originally maintained by Mustek
   Author:Jack Roy 2005.5.24

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

*/
#include <stdio.h>

#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#include "../include/typedef.h"
#include "../include/ini.h"

#ifndef PATH_MAX
# define PATH_MAX	1024
#endif

//#define HAVE_OS2_H

#if defined(HAVE_OS2_H)
# define DIR_SEP	";"
# define PATH_SEP	'\\'
#else
# define DIR_SEP	":"
# define PATH_SEP	'/'
#endif

/** @hideinitializer */
#define STRINGIFY1(x)	#x
/** @hideinitializer */
#define STRINGIFY(x)	STRINGIFY1(x)

#define DEFAULT_DIRS	"." DIR_SEP STRINGIFY(PATH_SANE_CONFIG_DIR)

char *
strsep (char **stringp, const char *delim)
{
	char *begin, *end;
	
	begin = *stringp;
	if (! begin || *begin == '\0')
		return NULL;
	
	/* Find the end of the token.  */
	end = strpbrk (begin, delim);
	if (end)
    {
		/* Terminate the token and set *STRINGP past NUL character.  */
		*end++ = '\0';
		*stringp = end;
    }
	else
		/* No more delimiters; this is the last token.  */
		*stringp = NULL;
	
	return begin;
}

FILE *
ini_open (const char *filename)
{
	char  *next, *dir, result[PATH_MAX];
	FILE *fp = 0;
	size_t len;
	void *mem = 0;
	char* dir_list=0;


	if (!dir_list)
    {
		dir_list = getenv ("SANE_CONFIG_DIR");
		if (dir_list)
		{
			len = strlen (dir_list);
			if ((len > 0) && (dir_list[len - 1] == DIR_SEP[0]))
			{
				/* append default search directories: */
				mem = malloc (len + sizeof (DEFAULT_DIRS));
				
				memcpy (mem, dir_list, len);
				memcpy ((char *) mem + len, DEFAULT_DIRS, sizeof (DEFAULT_DIRS));
				dir_list = mem;
			}
		}
		else
		{
			return NULL;
		}
    }

	for (next = dir_list; (dir = strsep (&next, DIR_SEP)) != 0; )
    {
		sprintf (result ,"%s%c%s", dir, PATH_SEP, filename);
		fp = fopen (result, "r");
		if (fp)
		{
		  break;
		}
    }
	
	if (mem)
		free(mem);
	
	return fp;
}

/** GET OPTION FROM ini FILE
 **/
 void  Trim( char * SrcStr )
{
  int i,j,k;

  i = j = k = 0;
  for( ; *(SrcStr+i) == ' ' ; i++ ) {;}
  for( j=strlen(SrcStr)-1 ; *(SrcStr+j) == ' '|| *(SrcStr+j)=='\n'|| *(SrcStr+j) == '\0'; j-- ) {;}
  for( k=i ; k<=j ; k++ )
      *(SrcStr+k-i) = *(SrcStr+k);
  
  *(SrcStr+j-i+1) = '\0';

  return;
}

/*read string*/
int OptionReadString(char * lpchTag, char * lpchTarItem , char* Def_Value, char*buf, int len,char* IniFileName)
{
  char * lpchOption = NULL;
  char * lpchSrcItem = NULL;
  char chLine[256];
  char * lpchLine = NULL;
  FILE * lpfp = NULL;
  int FoundTag;
	  
  lpfp = fopen(IniFileName, "r+");
  if( lpfp == NULL )
  {
      memcpy(buf,Def_Value,len);
	  return INI_NOFILE;
  }
   FoundTag = FALSE;
  while(!feof(lpfp))
  {
      fgets(chLine, 256, lpfp );
	 
	  
	  if(!feof(lpfp))
      {
		  Trim(chLine);
		   
        if(chLine[0] == '#')continue;
		  
		  if(FoundTag == FALSE) 
          { 
			  if(strcmp(chLine,lpchTag) == 0) 
				  {
					  FoundTag = TRUE;
				  } 
          }
		  else if(FoundTag == TRUE)
		  {
		  lpchLine = strtok(chLine, "=");
			  lpchSrcItem = lpchLine;
			  do {
				  lpchLine = strtok(NULL, "=");
				  if(lpchLine) { lpchOption = lpchLine; }
			  } while(lpchLine);
			  
			  if (lpchSrcItem != NULL && lpchOption != NULL)
			  {
				  if(strcmp(lpchSrcItem, lpchTarItem) ==0)
				  {
			          memcpy(buf,lpchOption,len);
					  fclose(lpfp);
					  return INI_DONE;
					  
				  }
			  }
			  
			  lpchLine = NULL;
			  lpchOption = NULL;
		  }
      }
  }
  memcpy(buf,Def_Value,len);
  fclose( lpfp );
  return INI_DONE;
}


int OptionReadInt(char * lpchTag, char * lpchTarItem ,int Def_Value ,char* IniFileName)
{
	char * lpchOption = NULL;
	char * lpchSrcItem = NULL;
	char chLine[256];
	char * lpchLine = NULL;
	FILE * lpfp = NULL;
	int FoundTag;

	lpfp = fopen(IniFileName, "r+");
  if( lpfp == NULL ) 
	{
		return Def_Value;
	}
	
	FoundTag = FALSE;
	while(!feof(lpfp))
	{
		fgets(chLine, 256, lpfp );
		if(!feof(lpfp))
		{
			Trim(chLine);

			if(chLine[0] == '#')continue;

			if(FoundTag == FALSE) 
			{ 
				if(strcmp(lpchTag, chLine)==0) { FoundTag = TRUE; } 
			}
			else if(FoundTag == TRUE)
			{
				lpchLine = strtok(chLine, "=");
				lpchSrcItem = lpchLine;
				do {
					lpchLine = strtok(NULL, "=");
					if(lpchLine) { lpchOption = lpchLine; }
				} while(lpchLine);
				
				if (lpchSrcItem != NULL && lpchOption != NULL)
				{
					if(strcmp(lpchSrcItem, lpchTarItem)==0)
					{
					
					    int temp = atoi(lpchOption);
						
                        fclose(lpfp);
						return temp;
						
					}
				}
				
				lpchLine = NULL;
				lpchOption = NULL;
			}
		}
	}
	fclose( lpfp );
	return Def_Value;
}

