/*$Header: mustek_a2nu2.c,v0.1 2005.5.23 vinci Exp $*/
/* sane - Scanner Access Now Easy.

   Copyright (C) 2000 Mustek.
   Originally maintained by Mustek

   Copyright (C) 2001, 2002 by Henning Meier-Geinitz.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB and similar 
   USB flatbed scanners.  */

#define BUILD 3

#include "../include/sane/config.h"

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>


#include "../include/sane/sanei.h"
#include "../include/sane/saneopts.h"

#define BACKEND_NAME mustek_a2nu2

#include "../include/sane/sanei_backend.h"

#include "a2nu2/include/debug.h"

#include "mustek_a2nu2.h"

#ifndef SANE_I18N
#define SANE_I18N(text) text
#endif

static SANE_Int num_devices;
static const SANE_Device **devlist = 0;
SANE_Bool m_bDSOpen = SANE_FALSE;              
SANE_Bool m_bIsScanning = SANE_FALSE;          
SANE_Bool m_bIsReading = SANE_FALSE;         

static SETPARAMETERS setpara;                  
static GETPARAMETERS getpara;                  
#define DEF_LINEARTTHRESHOLD 128

static SANE_Byte* Scan_data_buf;                /*store Scanned data for transfer*/
static SANE_Byte* Scan_data_buf_start;          /*point to data need to transfer*/
static size_t scan_buffer_len ;                 /* length of data buf*/
SANE_Word read_rows;                     /* transfer image's lines*/

SANE_Word *dpi_list;                     /*Resolution Support */


/* RIE: return if error */
#define RIE(function) do {status = function; if (status != SANE_STATUS_GOOD) \
return status;} while (SANE_FALSE)

#define PER_ADD_START_LINES 0
#define PRE_ADD_START_X 0
static const SANE_Range u8_range = {
  0,/* minimum */
  255,/* maximum */
  0/* quantization */
};
static SANE_Range x_range = {
  SANE_FIX (0.0),/* minimum */
  SANE_FIX (8.3*MM_PER_INCH),/* maximum */
  SANE_FIX (0.0)/* quantization */
};

static SANE_Range y_range = {
  SANE_FIX (0.0),/* minimum */
  SANE_FIX (11.6*MM_PER_INCH),/* maximum */
  SANE_FIX (0.0)/* quantization */
};

static SANE_Range gamma_range = {
  SANE_FIX (0.01),/* minimum */
  SANE_FIX (5.0),/* maximum */
  SANE_FIX (0.01)/* quantization */
};
static SANE_String_Const mode_list[] = {  
  SANE_I18N ("Color48"),       
  SANE_I18N ("Color24"),        
  SANE_I18N ("Gray16"),        
  SANE_I18N ("Gray8"),         
  SANE_I18N ("Lineart"),      
  0
};

static SANE_String_Const negative_mode_list[] = {
  SANE_I18N ("Color24"),      
  0
};

static SANE_String_Const source_list[] = {
    SANE_I18N ("Reflective"),
    SANE_I18N ("Positive"),
    SANE_I18N ("Negative"),
    0
};
static Scanner_Model mustek_A2nu2_model = {
  "mustek-A2nu2",/* Name */
  "Mustek",/* Device vendor string */

  "BearPaw 2448TA Pro",/* Device model name */
   ""  ,/* Name of the firmware file */
   
  {1200, 600, 300, 150, 75, 0},/* possible resolutions */
  
  SANE_FIX (0.0),		/* Start of scan area in mm  (x) */
  SANE_FIX (0.0),		/* Start of scan area in mm (y) */
  SANE_FIX (8.3*MM_PER_INCH),	/* Size of scan area in mm (x) */
  SANE_FIX (11.6*MM_PER_INCH),		/* Size of scan area in mm (y) */

  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
  SANE_FIX (1.46*MM_PER_INCH),	/* Size of scan area in TA mode in mm (x) */
  SANE_FIX (6.45*MM_PER_INCH),		/* Size of scan area in TA mode in mm (y) */

  0,            /* Order of the CCD/CIS colors 0:RO_RGB 1:RO_BGR*/
  SANE_FIX (2.0),/* Default gamma value */

  SANE_FALSE,/* Is this a CIS scanner? */
  0     /* Which flags are needed for this scanner? */
  /* Setup and tested */
};

/*-----------------------------------*/

static size_t
max_string_size (const SANE_String_Const strings[])
{
  size_t size, max_size = 0;
  SANE_Int i;

  for (i = 0; strings[i]; ++i)
  {
    size = strlen (strings[i]) + 1;
    if (size > max_size)
      max_size = size;
  }
  return max_size;
}

static SANE_Status  calc_parameters(Mustek_Scanner * s)
{
  SANE_String val,val_source;
  val = s->val[OPT_MODE].s;
  val_source = s->val[OPT_SOURCE].s;
  
  s->params.last_frame = SANE_TRUE;

  if(strcmp (val, "Color48") == 0)			/* Color48 */
  {
    s->params.format = SANE_FRAME_RGB;
    s->params.depth =16;
    setpara.smScanMode = SM_RGB48;
    if(s->val[OPT_PREVIEW].w)
    {
      XDBG(DEBUG_SANE,"calc_parameters : preview set ScanMode SM_RGB24");
      s->params.depth = 8;
      setpara.smScanMode = SM_RGB24;
    }
  } 
  else if(strcmp (val, "Color24") == 0)			/* Color24 */
  {
    s->params.format = SANE_FRAME_RGB;
    s->params.depth = 8;
    setpara.smScanMode = SM_RGB24;
  } 
  else if (strcmp (val, "Gray16") == 0 )
  {
    s->params.format = SANE_FRAME_GRAY;
    s->params.depth = 16;
    setpara.smScanMode = SM_GRAY16;
    if(s->val[OPT_PREVIEW].w)
    {
      s->params.depth = 8;
      XDBG(DEBUG_SANE,"calc_parameters : preview set ScanMode SM_GRAY");
      setpara.smScanMode = SM_GRAY;
    }
  }
  else if (strcmp (val, "Gray8") == 0 )
  {
    s->params.format = SANE_FRAME_GRAY;
    s->params.depth = 8;
    setpara.smScanMode = SM_GRAY;
  }
  else if(strcmp (val, "Lineart") == 0)
  {
    s->params.format = SANE_FRAME_GRAY;
    s->params.depth = 1;
    setpara.smScanMode = SM_TEXT;
  }

  /*set Scan Source*/
  XDBG(DEBUG_SANE,"calc_parameters :scan Source = %s ",val_source);
  if( strcmp (val_source,"Reflective")==0)
  {
    setpara.ssScanSource =SS_Reflective;
  }
  else if(strcmp (val_source,"Positive")==0)
  {
    setpara.ssScanSource =SS_Positive;
  }
  else if(strcmp (val_source,"Negative")==0)
  {
    setpara.ssScanSource =SS_Negative;
  }
    

  setpara.fmArea.x1 = (WORD)((SANE_UNFIX(s->val[OPT_TL_X].w)*300.0)/MM_PER_INCH+0.5) ;        
  setpara.fmArea.x2 = (WORD)((SANE_UNFIX(s->val[OPT_BR_X].w)*300.0)/MM_PER_INCH+0.5) ; 		  
  setpara.fmArea.y1 = (WORD)((SANE_UNFIX(s->val[OPT_TL_Y].w)*300.0)/MM_PER_INCH+0.5) ; 		  
  setpara.fmArea.y2 = (WORD)((SANE_UNFIX(s->val[OPT_BR_Y].w)*300.0)/MM_PER_INCH+0.5) ;       

  if(s->val[OPT_PREVIEW].w)
  {
    setpara.fmArea.y1=setpara.fmArea.y1+PER_ADD_START_LINES;
    setpara.fmArea.x1+=PRE_ADD_START_X;
  } /*just for range bug.*/

  setpara.pfPixelFlavor = PF_BlackIs0;
  setpara.wLinearThreshold =s->val[OPT_THRESHOLD].w ;

  setpara.wTargetDPI = s->val[OPT_RESOLUTION].w;
  if(s->val[OPT_PREVIEW].w)
  {
    setpara.wTargetDPI = 75;
  }

  setpara.pGammaTable=NULL;

  s->params.pixels_per_line = (SANE_Int)((setpara.fmArea.x2 - setpara.fmArea.x1)*setpara.wTargetDPI/300.0+0.5);

  switch(s->params.format) {
  case SANE_FRAME_RGB:
    if(s->params.depth == 8 )
      s->params.bytes_per_line =s->params.pixels_per_line*3;
    if(s->params.depth == 16 )
    s->params.bytes_per_line =s->params.pixels_per_line*6;
    break;
  case SANE_FRAME_GRAY:
    if(s->params.depth == 1 )
      s->params.bytes_per_line =s->params.pixels_per_line/8;
    if(s->params.depth == 8 )
      s->params.bytes_per_line =s->params.pixels_per_line;
    if(s->params.depth == 16 )
      s->params.bytes_per_line =s->params.pixels_per_line*2;
    break;
  default:
    XDBG(DEBUG_SANE,"sane_star:sane params .format = %d",s->params.format);
  }

  s->params.lines=(SANE_Int)( (setpara.fmArea.y2 - setpara.fmArea.y1)*setpara.wTargetDPI/300+0.5);

  XDBG(DEBUG_SANE,"sane_star:Exit");

  return SANE_STATUS_GOOD;
}

static SANE_Status
init_options (Mustek_Scanner * s)
{
  SANE_Int option, count;

  XDBG(DEBUG_SANE, "init_options: start");

  memset (s->opt, 0, sizeof (s->opt));
  memset (s->val, 0, sizeof (s->val));

  for (option = 0; option < NUM_OPTIONS; ++option)
  {
    s->opt[option].size = sizeof (SANE_Word);
    s->opt[option].cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
  }
  /*  Option num*/
  s->opt[OPT_NUM_OPTS].name = SANE_NAME_NUM_OPTIONS;
  s->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
  s->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
  s->opt[OPT_NUM_OPTS].type = SANE_TYPE_INT;
  s->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
  s->val[OPT_NUM_OPTS].w = NUM_OPTIONS;

  /* "Mode" group: */
  s->opt[OPT_MODE_GROUP].title = SANE_I18N ("Scan Mode");
  s->opt[OPT_MODE_GROUP].desc = "";
  s->opt[OPT_MODE_GROUP].type = SANE_TYPE_GROUP;
  s->opt[OPT_MODE_GROUP].size = 0;
  s->opt[OPT_MODE_GROUP].cap = 0;
  s->opt[OPT_MODE_GROUP].constraint_type = SANE_CONSTRAINT_NONE;

  /* scan mode */
  s->opt[OPT_MODE].name = SANE_NAME_SCAN_MODE;
  s->opt[OPT_MODE].title = SANE_TITLE_SCAN_MODE;
  s->opt[OPT_MODE].desc = SANE_DESC_SCAN_MODE;
  s->opt[OPT_MODE].type = SANE_TYPE_STRING;
  s->opt[OPT_MODE].constraint_type = SANE_CONSTRAINT_STRING_LIST;
  s->opt[OPT_MODE].size = max_string_size (mode_list);
  s->opt[OPT_MODE].constraint.string_list = mode_list;
  s->val[OPT_MODE].s = strdup ("Color24");

  /* Scan Source*/
  s->opt[OPT_SOURCE].name = SANE_NAME_SCAN_SOURCE;
  s->opt[OPT_SOURCE].title = SANE_TITLE_SCAN_SOURCE;
  s->opt[OPT_SOURCE].desc = SANE_DESC_SCAN_SOURCE;
  s->opt[OPT_SOURCE].type = SANE_TYPE_STRING;
  s->opt[OPT_SOURCE].constraint_type = SANE_CONSTRAINT_STRING_LIST;
  s->opt[OPT_SOURCE].size = max_string_size (source_list);
  s->opt[OPT_SOURCE].constraint.string_list =source_list;
  s->val[OPT_SOURCE].s = strdup ("Reflective");

  if(!IsTAConnected())
  {
    DISABLE (OPT_SOURCE);
  }


  /* resolution */

  for (count = 0; s->model.dpi_values[count] != 0; count++)
  {}
  dpi_list = malloc ((count + 1) * sizeof (SANE_Word));
  if (!dpi_list)
  return SANE_STATUS_NO_MEM;
  dpi_list[0] = count;

  for (count = 0; s->model.dpi_values[count] != 0; count++)
    dpi_list[count + 1] = s->model.dpi_values[count];

  s->opt[OPT_RESOLUTION].name = SANE_NAME_SCAN_RESOLUTION;
  s->opt[OPT_RESOLUTION].title = SANE_TITLE_SCAN_RESOLUTION;


  s->opt[OPT_RESOLUTION].desc = SANE_DESC_SCAN_RESOLUTION;
  s->opt[OPT_RESOLUTION].type = SANE_TYPE_INT;
  s->opt[OPT_RESOLUTION].unit = SANE_UNIT_DPI;
  s->opt[OPT_RESOLUTION].constraint_type = SANE_CONSTRAINT_WORD_LIST;
  s->opt[OPT_RESOLUTION].constraint.word_list = dpi_list;
  s->val[OPT_RESOLUTION].w = 300;

  /* preview */
  s->opt[OPT_PREVIEW].name = SANE_NAME_PREVIEW;
  s->opt[OPT_PREVIEW].title = SANE_TITLE_PREVIEW;
  s->opt[OPT_PREVIEW].desc = SANE_DESC_PREVIEW;
  s->opt[OPT_PREVIEW].cap = SANE_CAP_SOFT_DETECT | SANE_CAP_SOFT_SELECT;
  s->opt[OPT_PREVIEW].type = SANE_TYPE_BOOL;
  s->val[OPT_PREVIEW].w = SANE_FALSE;

  /* "Debug" group: */
  s->opt[OPT_DEBUG_GROUP].title = SANE_I18N ("Debugging Options");
  s->opt[OPT_DEBUG_GROUP].desc = "";
  s->opt[OPT_DEBUG_GROUP].type = SANE_TYPE_GROUP;
  s->opt[OPT_DEBUG_GROUP].size = 0;
  s->opt[OPT_DEBUG_GROUP].cap = 0;
  s->opt[OPT_DEBUG_GROUP].constraint_type = SANE_CONSTRAINT_NONE;

  /* auto warmup */
  s->opt[OPT_AUTO_WARMUP].name = "auto-warmup";
  s->opt[OPT_AUTO_WARMUP].title = SANE_I18N ("Automatic warmup");
  s->opt[OPT_AUTO_WARMUP].desc =
  SANE_I18N ("Warm-up until the lamp's brightness is constant "
           "instead of insisting on 40 seconds warm-up time.");
  s->opt[OPT_AUTO_WARMUP].type = SANE_TYPE_BOOL;
  s->opt[OPT_AUTO_WARMUP].unit = SANE_UNIT_NONE;
  s->opt[OPT_AUTO_WARMUP].constraint_type = SANE_CONSTRAINT_NONE;
  s->val[OPT_AUTO_WARMUP].w = SANE_FALSE;
  if(s->model.is_cis)
    DISABLE (OPT_AUTO_WARMUP);

  /* "Enhancement" group: */
  s->opt[OPT_ENHANCEMENT_GROUP].title = SANE_I18N ("Enhancement");
  s->opt[OPT_ENHANCEMENT_GROUP].desc = "";
  s->opt[OPT_ENHANCEMENT_GROUP].type = SANE_TYPE_GROUP;

  s->opt[OPT_ENHANCEMENT_GROUP].size = 0;
  s->opt[OPT_ENHANCEMENT_GROUP].cap = 0;
  s->opt[OPT_ENHANCEMENT_GROUP].constraint_type = SANE_CONSTRAINT_NONE;

  /* threshold */
  s->opt[OPT_THRESHOLD].name = SANE_NAME_THRESHOLD;
  s->opt[OPT_THRESHOLD].title = SANE_TITLE_THRESHOLD;
  s->opt[OPT_THRESHOLD].desc = SANE_DESC_THRESHOLD;
  s->opt[OPT_THRESHOLD].type = SANE_TYPE_INT;
  s->opt[OPT_THRESHOLD].unit = SANE_UNIT_NONE;
  s->opt[OPT_THRESHOLD].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_THRESHOLD].constraint.range = &u8_range;
  s->opt[OPT_THRESHOLD].cap |= SANE_CAP_INACTIVE;
  s->val[OPT_THRESHOLD].w = DEF_LINEARTTHRESHOLD ;

  /* internal gamma value */
  s->opt[OPT_GAMMA_VALUE].name = "gamma-value";
  s->opt[OPT_GAMMA_VALUE].title = SANE_I18N ("Gamma value");
  s->opt[OPT_GAMMA_VALUE].desc = SANE_I18N ("Sets the gamma value of all channels.");
  s->opt[OPT_GAMMA_VALUE].type = SANE_TYPE_FIXED;
  s->opt[OPT_GAMMA_VALUE].unit = SANE_UNIT_NONE;
  s->opt[OPT_GAMMA_VALUE].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_GAMMA_VALUE].constraint.range = &gamma_range;
  s->opt[OPT_GAMMA_VALUE].cap |= SANE_CAP_EMULATED;
  s->val[OPT_GAMMA_VALUE].w = s->model.default_gamma_value;

  DISABLE (OPT_GAMMA_VALUE);

  /* "Geometry" group: */
  s->opt[OPT_GEOMETRY_GROUP].title = SANE_I18N ("Geometry");
  s->opt[OPT_GEOMETRY_GROUP].desc = "";
  s->opt[OPT_GEOMETRY_GROUP].type = SANE_TYPE_GROUP;
  s->opt[OPT_GEOMETRY_GROUP].cap = SANE_CAP_ADVANCED;
  s->opt[OPT_GEOMETRY_GROUP].size = 0;
  s->opt[OPT_GEOMETRY_GROUP].constraint_type = SANE_CONSTRAINT_NONE;

  x_range.max = s->model.x_size;
  y_range.max = s->model.y_size;

  /* top-left x */
  s->opt[OPT_TL_X].name = SANE_NAME_SCAN_TL_X;
  s->opt[OPT_TL_X].title = SANE_TITLE_SCAN_TL_X;
  s->opt[OPT_TL_X].desc = SANE_DESC_SCAN_TL_X;
  s->opt[OPT_TL_X].type = SANE_TYPE_FIXED;
  s->opt[OPT_TL_X].unit = SANE_UNIT_MM;
  s->opt[OPT_TL_X].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_TL_X].constraint.range = &x_range;

  s->val[OPT_TL_X].w = 0;

  /* top-left y */
  s->opt[OPT_TL_Y].name = SANE_NAME_SCAN_TL_Y;
  s->opt[OPT_TL_Y].title = SANE_TITLE_SCAN_TL_Y;
  s->opt[OPT_TL_Y].desc = SANE_DESC_SCAN_TL_Y;
  s->opt[OPT_TL_Y].type = SANE_TYPE_FIXED;
  s->opt[OPT_TL_Y].unit = SANE_UNIT_MM;
  s->opt[OPT_TL_Y].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_TL_Y].constraint.range = &y_range;
  s->val[OPT_TL_Y].w = 0;

  /* bottom-right x */
  s->opt[OPT_BR_X].name = SANE_NAME_SCAN_BR_X;
  s->opt[OPT_BR_X].title = SANE_TITLE_SCAN_BR_X;
  s->opt[OPT_BR_X].desc = SANE_DESC_SCAN_BR_X;
  s->opt[OPT_BR_X].type = SANE_TYPE_FIXED;
  s->opt[OPT_BR_X].unit = SANE_UNIT_MM;
  s->opt[OPT_BR_X].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_BR_X].constraint.range = &x_range;
  s->val[OPT_BR_X].w = x_range.max;

  /* bottom-right y */
  s->opt[OPT_BR_Y].name = SANE_NAME_SCAN_BR_Y;
  s->opt[OPT_BR_Y].title = SANE_TITLE_SCAN_BR_Y;
  s->opt[OPT_BR_Y].desc = SANE_DESC_SCAN_BR_Y;
  s->opt[OPT_BR_Y].type = SANE_TYPE_FIXED;
  s->opt[OPT_BR_Y].unit = SANE_UNIT_MM;
  s->opt[OPT_BR_Y].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_BR_Y].constraint.range = &y_range;
  s->val[OPT_BR_Y].w = y_range.max;

  calc_parameters(s);

  XDBG(DEBUG_SANE,"init_options: exit");
  return SANE_STATUS_GOOD;
}

SANE_Status
sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
{ 
  num_devices = 1;   /*only one device in this backend*/
  XDBG(DEBUG_SANE,"sane_init:");
  XDBG(DEBUG_SANE, "SANE Mustek USB backend version %d.%d build %d from %s", V_MAJOR,
      V_MINOR, BUILD, PACKAGE_STRING);

    if ( version_code!= NULL)
       *version_code = SANE_VERSION_CODE ( V_MAJOR, V_MINOR, BUILD);
    XDBG(DEBUG_SANE, "sane_init: authorize %s null", authorize ? "!=" : "==");
   

    return SANE_STATUS_GOOD;
}

void sane_exit (void)
{
   XDBG(DEBUG_SANE,"sane_exit: start");
	
	if (devlist != NULL)
	{
		free (devlist);
		devlist = NULL;
	}
	if(Scan_data_buf != NULL)
		free(Scan_data_buf);

	Scan_data_buf=NULL;
	
	devlist = NULL;
    XDBG(DEBUG_SANE,"%s", "sane_exit: end");
}

SANE_Status
sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
{
	SANE_Int dev_num;
	XDBG(DEBUG_SANE,"sane_get_devices :call in ");
	XDBG(DEBUG_SANE, "sane_get_devices: start: local_only = %s",
		local_only == SANE_TRUE ? "true" : "false");
	
	if (devlist!= NULL)
	{
		free (devlist);
		devlist = NULL;
	}

	devlist = malloc ((num_devices + 1) * sizeof (devlist[0]));
	if (devlist ==NULL)
		return SANE_STATUS_NO_MEM;
	
	dev_num = 0;
	if(GetDeviceStatus())
	{  
		SANE_Device *sane_device;

      sane_device = malloc (sizeof (*sane_device));
      if (sane_device == NULL)
		return SANE_STATUS_NO_MEM;
      sane_device->name = strdup("BearPaw 2448 TA Pro");
      sane_device->vendor = strdup("Mustek");
      sane_device->model = strdup("Mustek");
      sane_device->type = strdup ("flatbed scanner");
      devlist[dev_num++] = sane_device;
	}
	devlist[dev_num]=0;
	*device_list = devlist;
	XDBG(DEBUG_SANE, "sane_get_devices: exit");
	return SANE_STATUS_GOOD;
}

SANE_Status
sane_open (SANE_String_Const devicename, SANE_Handle * handle)
{
	Mustek_Scanner *s;
	XDBG(DEBUG_SANE,"sane open : call in");
	XDBG(DEBUG_SANE, "sane_open: start :devicename = %s", devicename);
	
	if(m_bDSOpen)
		return SANE_STATUS_COVER_OPEN;
		

    if(!_Open()) 
	{
		return SANE_STATUS_INVAL;
	}
	if(!PowerControl(SANE_FALSE,SANE_FALSE)) 
	{
		return SANE_STATUS_INVAL;
	}
	if(!CarriageHome())
	{
		return SANE_STATUS_INVAL;
	}

	s = malloc (sizeof (*s));
	if (s == NULL)
		return SANE_STATUS_NO_MEM;
	memset (s, 0, sizeof (*s));

	s->gamma_table=NULL;
	memcpy(&s->model,&mustek_A2nu2_model,sizeof(Scanner_Model));
	s->next = NULL;

	init_options (s);
	*handle = s;
	m_bDSOpen =SANE_TRUE;
	
	read_rows = 0 ;
	scan_buffer_len = 0;

	return SANE_STATUS_GOOD;
}


void
sane_close (SANE_Handle handle)
{
    XDBG(DEBUG_SANE, "sane_close: start");
    
	m_bDSOpen = SANE_FALSE;
    
	PowerControl(SANE_FALSE,SANE_FALSE);  
    
	CarriageHome();

	_Close();
	
	free (handle);  

	
	XDBG(DEBUG_SANE, "sane_close: exit");
}


const SANE_Option_Descriptor *
sane_get_option_descriptor (SANE_Handle handle, SANE_Int option)
{
	Mustek_Scanner *s = handle;
	
	if ((unsigned) option >= NUM_OPTIONS)
		return 0;
	XDBG(DEBUG_SANE, "sane_get_option_descriptor: option = %s (%d)",
		s->opt[option].name, option);
	return s->opt + option;
}


SANE_Status
sane_control_option (SANE_Handle handle, SANE_Int option,
		     SANE_Action action, void *val, SANE_Int * info)
{
	Mustek_Scanner *s = handle;
	SANE_Status status;
	SANE_Word cap;
	SANE_Int myinfo = 0;
	
	XDBG(DEBUG_SANE, "sane_control_option: start: action = %s, option = %s (%d)",
		(action == SANE_ACTION_GET_VALUE) ? "get" :
	(action == SANE_ACTION_SET_VALUE) ? "set" :
	(action == SANE_ACTION_SET_AUTO) ? "set_auto" : "unknown",
		s->opt[option].name, option);

	
	if (info)
		*info = 0;
	
	if (m_bIsScanning)
    {
		XDBG(DEBUG_SANE, "sane_control_option: don't call this function while "
			"scanning");
		return SANE_STATUS_DEVICE_BUSY;
    }
	if (option >= NUM_OPTIONS || option < 0)
    {
		XDBG(DEBUG_SANE, "sane_control_option: option %d >= NUM_OPTIONS || option < 0",
			option);
		return SANE_STATUS_INVAL;
    }
	
	cap = s->opt[option].cap;
	if (!SANE_OPTION_IS_ACTIVE (cap))
    {
		XDBG(DEBUG_SANE, "sane_control_option: option %d is inactive", option);
		return SANE_STATUS_INVAL;
    }
	if (action == SANE_ACTION_GET_VALUE)
    {
		switch (option)
		{
			/* word options: */
		case OPT_NUM_OPTS:
		case OPT_RESOLUTION:
		case OPT_PREVIEW:
		case OPT_AUTO_WARMUP:
		case OPT_GAMMA_VALUE:
		case OPT_THRESHOLD:
		case OPT_TL_X:
		case OPT_TL_Y:
		case OPT_BR_X:
		case OPT_BR_Y:
			*(SANE_Word *) val = s->val[option].w;
			break;
			/* string options: */
		case OPT_MODE:
			strcpy (val, s->val[option].s);
			break;

		case OPT_SOURCE:
			strcpy (val, s->val[option].s);
			break;
		default:
			XDBG(DEBUG_SANE, "sane_control_option: can't get unknown option %d",
				option);
			;
		}
    }
	else if (action == SANE_ACTION_SET_VALUE)
    {
		if (!SANE_OPTION_IS_SETTABLE (cap))
		{
			XDBG(DEBUG_SANE, "sane_control_option: option %d is not settable", option);
			return SANE_STATUS_INVAL;
		}
		

		
		switch (option)
		{
			/* (mostly) side-effect-free word options: */
		case OPT_RESOLUTION:
		case OPT_PREVIEW:
		case OPT_TL_X:
		case OPT_TL_Y:
		case OPT_BR_X:
		case OPT_BR_Y:
			s->val[option].w = *(SANE_Word *) val;
			RIE (calc_parameters (s));
			myinfo |= SANE_INFO_RELOAD_PARAMS;

			break;
		case OPT_THRESHOLD:
		case OPT_AUTO_WARMUP:

		case OPT_GAMMA_VALUE:
			s->val[option].w = *(SANE_Word *) val;
			break;
			/* side-effect-free word-array options: */      
		case OPT_MODE:
			if (s->val[option].s)
				free (s->val[option].s);
			s->val[option].s = strdup (val);
			if (strcmp (s->val[option].s, "Lineart") == 0)
			{
				ENABLE (OPT_THRESHOLD);
			}
			else
			{
				DISABLE (OPT_THRESHOLD);
			}
			RIE (calc_parameters (s));
			myinfo |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
			break;
		case OPT_SOURCE:
			if (strcmp (s->val[option].s, val) != 0)
			{			/* something changed */
				if (s->val[option].s)
					free (s->val[option].s);
				s->val[option].s = strdup (val);
				if (strcmp (s->val[option].s, "Reflective") == 0)
				{
					PowerControl(SANE_TRUE,SANE_FALSE);
          s->opt[OPT_MODE].size = max_string_size (mode_list);
          s->opt[OPT_MODE].constraint.string_list = mode_list;
          s->val[OPT_MODE].s = strdup ("Color24");
				    x_range.max = s->model.x_size;
				    y_range.max = s->model.y_size;
				}
				else if(0 == strcmp (s->val[option].s,  "Negative"))
				{
          PowerControl(SANE_FALSE, SANE_TRUE);
          s->opt[OPT_MODE].size = max_string_size (negative_mode_list);
          s->opt[OPT_MODE].constraint.string_list = negative_mode_list;
          s->val[OPT_MODE].s = strdup ("Color24");
          x_range.max = s->model.x_size_ta;
          y_range.max = s->model.y_size_ta;
      }
      else if(0 == strcmp (s->val[option].s,  "Positive"))
      {
          PowerControl(SANE_FALSE, SANE_TRUE);
          s->opt[OPT_MODE].size = max_string_size (mode_list);
          s->opt[OPT_MODE].constraint.string_list = mode_list;
          s->val[OPT_MODE].s = strdup ("Color24");         
          x_range.max = s->model.x_size_ta;
          y_range.max = s->model.y_size_ta;      
      }	
			}
			myinfo |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
			break;
		default:
			XDBG(DEBUG_SANE, "sane_control_option: can't set unknown option %d",
			option);
		}
    }
	else
    {
		XDBG(DEBUG_SANE,"sane_control_option: unknown action %d for option %d",
			action, option);
		return SANE_STATUS_INVAL;
    }
	if (info)
		*info = myinfo;
	
	XDBG(DEBUG_SANE, "sane_control_option: exit");
	return SANE_STATUS_GOOD;
}

SANE_Status
sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
{
  Mustek_Scanner *s = handle;
  	
  XDBG(DEBUG_SANE, "sane_get_parameters: start");

  XDBG(DEBUG_SANE,"sane_get_parameters :params.format = %d ",s->params.format);

  XDBG(DEBUG_SANE,"sane_get_parameters :params.depth = %d ",s->params.depth);
  XDBG(DEBUG_SANE,"sane_get_parameters :params.pixels_per_line = %d ",s->params.pixels_per_line);
  XDBG(DEBUG_SANE,"sane_get_parameters :params.bytes_per_line = %d ",s->params.bytes_per_line);
  XDBG(DEBUG_SANE,"sane_get_parameters :params.lines = %d ",s->params.lines);
  if ( params!= NULL)
    *params = s->params;

  XDBG(DEBUG_SANE, "sane_get_parameters: exit");

  return SANE_STATUS_GOOD;
		
}

SANE_Status
sane_start (SANE_Handle handle)
{
	int i;
	Mustek_Scanner *s = handle;

	XDBG(DEBUG_SANE, "sane_start: start");
	
	scan_buffer_len = 0;
	
	calc_parameters(s);
	
	if (s->val[OPT_TL_X].w >= s->val[OPT_BR_X].w)
    {
		XDBG(DEBUG_SANE, "sane_start: top left x >= bottom right x --- exiting");
		return SANE_STATUS_INVAL;
    }
	if (s->val[OPT_TL_Y].w >= s->val[OPT_BR_Y].w)
    {
		XDBG(DEBUG_SANE, "sane_start: top left y >= bottom right y --- exiting");
		return SANE_STATUS_INVAL;
    }

	setpara.pGammaTable = NULL;
	
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.fmArea.x1=%d ",setpara.fmArea.x1);
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.fmArea.x2=%d ",setpara.fmArea.x2);
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.fmArea.y1=%d ",setpara.fmArea.y1);
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.fmArea.y2=%d ",setpara.fmArea.y2);
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.pfPixelFlavor=%d ",setpara.pfPixelFlavor);
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.wLinearThreshold=%d ",setpara.wLinearThreshold);
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.wTargetDPI=%d ",setpara.wTargetDPI);
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.smScanMode=%d ",setpara.smScanMode);
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.ssScanSource =%d ",setpara.ssScanSource);
	XDBG(DEBUG_SANE,"Sane_start:setpara ,setpara.pGammaTable =%d ",setpara.pGammaTable);
	
		
	SetParameters(&setpara);
	
	GetParameters(&getpara);
		
	switch(s->params.format) {
	case SANE_FRAME_RGB:
		if(s->params.depth == 8 )

			s->params.pixels_per_line =getpara.dwLineByteWidth/3;
		if(s->params.depth == 16 )
			s->params.pixels_per_line =getpara.dwLineByteWidth/6;


		break;
	case SANE_FRAME_GRAY:
		if(s->params.depth == 1 )
			s->params.pixels_per_line =getpara.dwLineByteWidth*8;
		if(s->params.depth == 8 )
			s->params.pixels_per_line =getpara.dwLineByteWidth;
		if(s->params.depth == 16 )
			s->params.pixels_per_line =getpara.dwLineByteWidth/2;
		break;
	default:
		XDBG(DEBUG_SANE,"sane_star:sane params .format = %d",s->params.format);
	}
	
	s->params.bytes_per_line =getpara.dwLineByteWidth;
    s->params.lines= getpara.dwLength;

	s->params.last_frame = TRUE;
	
	
	read_rows= getpara.dwLength;
	XDBG(DEBUG_SANE,"sane_start : read_rows = %d",read_rows);

	/*warmming up*/
	if(s->val[OPT_AUTO_WARMUP].w){
	for (i=30 ;i>0 ;i--)
		{
			sleep(1);
			fprintf(stderr," warming up : %d \n",i);
		}	
    }
	fprintf(stderr," SCANNING ... \n");

	
	m_bIsScanning = SANE_TRUE;
	if(Scan_data_buf==NULL)
		free(Scan_data_buf);
	Scan_data_buf = NULL;
	
	Scan_data_buf = malloc(SCAN_BUFFER_SIZE*sizeof(SANE_Byte));
	if(Scan_data_buf==NULL)
		return SANE_STATUS_NO_MEM;
	
	StartScan();
	
	XDBG(DEBUG_SANE, "sane_start: exit");

	return SANE_STATUS_GOOD;
}

SANE_Status
sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
	   SANE_Int * len)
{

	Mustek_Scanner *s = handle;
	static BYTE* tempbuf;
	SANE_Int lines_to_read ,lines_read;
	IMAGEROWS image_row;

	int maxbuffersize = max_len;
	
	XDBG(DEBUG_SANE, "sane_read: start");
	XDBG(DEBUG_SANE,"sane_read: max_len=%d",max_len);
	
	if (s==NULL)
    {
		XDBG(DEBUG_SANE, "sane_read: handle is null!");
		return SANE_STATUS_INVAL;
    }

	if (buf==NULL)
    {
	    XDBG(DEBUG_SANE, "sane_read: buf is null!");
		return SANE_STATUS_INVAL;
    }
	
	if (len==NULL)
    {
		XDBG(DEBUG_SANE, "sane_read: len is null!");
		return SANE_STATUS_INVAL;
    }
	*len = 0; 
	if (!m_bIsScanning)
    {
		XDBG(DEBUG_SANE, "sane_read: scan was cancelled, is over or has not been "
			"initiated yet");
		return SANE_STATUS_CANCELLED;
    }
	XDBG(DEBUG_SANE, "sane_read: before read data read_row=%d",read_rows);
	if(scan_buffer_len ==0)
	{
		if(read_rows > 0)
		{	
			lines_to_read = SCAN_BUFFER_SIZE / getpara.dwLineByteWidth;
			
			if(lines_to_read > read_rows)
				lines_to_read = read_rows;
			
			tempbuf = (BYTE*)malloc(sizeof(BYTE)*lines_to_read*getpara.dwLineByteWidth+3*1024+1);
      memset(tempbuf,0,sizeof(BYTE)*lines_to_read*getpara.dwLineByteWidth+3*1024+1);
      
      XDBG(DEBUG_SANE,"sane_read: buffer size is %d", sizeof(BYTE)*lines_to_read*getpara.dwLineByteWidth+3*1024+1);
      
			image_row.roRgbOrder = mustek_A2nu2_model.line_mode_color_order;
			image_row.wWantedLineNum =lines_to_read;
			image_row.pBuffer = (LPBYTE) tempbuf;
			m_bIsReading = SANE_TRUE;
			
			if(!ReadScannedData(&image_row))
			{
				XDBG(DEBUG_SANE,"sane_read: ReadScannedData error");
				m_bIsReading = SANE_FALSE;
				return SANE_STATUS_INVAL;
			}
			
			XDBG(DEBUG_SANE,"sane_read: Finish ReadScanedData");
			m_bIsReading = SANE_FALSE;
			memset(Scan_data_buf,0,SCAN_BUFFER_SIZE);
			scan_buffer_len = image_row.wXferedLineNum*getpara.dwLineByteWidth;
			XDBG(DEBUG_SANE,"sane_read : scan_buffer_len = %d",scan_buffer_len);
			
			memcpy(Scan_data_buf,tempbuf,scan_buffer_len);

			XDBG(DEBUG_SANE,"sane_read :after memcpy");
			free(tempbuf);
			Scan_data_buf_start = Scan_data_buf;
			read_rows -= image_row.wXferedLineNum;

		}
		else
		{
			XDBG(DEBUG_SANE, "sane_read: scan finished -- exit");
			sane_cancel(handle);
			return SANE_STATUS_EOF;
		}
	}
	if(scan_buffer_len ==0 )
	{
		XDBG(DEBUG_SANE, "sane_read: scan finished -- exit");
		sane_cancel(handle);
		return SANE_STATUS_EOF;
	}




	lines_read=(maxbuffersize<(SANE_Int)scan_buffer_len)?maxbuffersize:(SANE_Int)scan_buffer_len;
	XDBG(DEBUG_SANE,"sane_read: after %d",lines_read);
	
	*len= (SANE_Int)lines_read;
	
	XDBG(DEBUG_SANE,"sane_read: after 2");
	
	XDBG(DEBUG_SANE,"sane_read : get lines_read = %d",lines_read);
	XDBG(DEBUG_SANE,"sane_read : get *len = %d",*len);
	memcpy(buf,Scan_data_buf_start,lines_read);
	
	scan_buffer_len -= lines_read;
	Scan_data_buf_start +=lines_read;
	XDBG(DEBUG_SANE,"sane_read :exit");
	return SANE_STATUS_GOOD;
	
}

void
sane_cancel (SANE_Handle handle)
{
	
    Mustek_Scanner *s = handle;
	int i;
	XDBG(DEBUG_SANE, "sane_cancel: in");
	if(m_bIsScanning)
	{
		m_bIsScanning= SANE_FALSE;
		if(read_rows > 0 )
		{
			XDBG(DEBUG_SANE,"sane_cancel: warning: is scanning");

		}
		else
		{
			XDBG(DEBUG_SANE,"sane_cancel: Scan finished");
		}
	
		StopScan();
		
		CarriageHome();
		for(i=0;i<20;i++)
		{
			if(m_bIsReading ==SANE_FALSE)
			{
				if (s->gamma_table != NULL)
				{
					free (s->gamma_table);
					s->gamma_table = NULL;
					break;
				}
			}
			else
				sleep(1);
		}
		if(Scan_data_buf != NULL)
		{	
			free(Scan_data_buf);
			Scan_data_buf = NULL;
			Scan_data_buf_start = NULL;
		}
		
		read_rows = 0;
		scan_buffer_len = 0;
		memset(&setpara,0,sizeof(setpara));
		memset(&getpara,0,sizeof(getpara));
		
	}
	else
	{
		XDBG(DEBUG_SANE,"sane_cancel: do nothing");
	}


	XDBG(DEBUG_SANE, "sane_cancel: exit");

}

SANE_Status
sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
{
	XDBG(DEBUG_SANE, "sane_set_io_mode: handle = %p, non_blocking = %s",
		handle, non_blocking == SANE_TRUE ? "true" : "false");
	if (!m_bIsScanning)
    {
		XDBG(DEBUG_SANE, "sane_set_io_mode: not scanning");
		return SANE_STATUS_INVAL;
    }
	if (non_blocking)
		return SANE_STATUS_UNSUPPORTED;
	return SANE_STATUS_GOOD;
}

SANE_Status
sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
{ 
	XDBG(DEBUG_SANE, "sane_get_select_fd: handle = %p, fd = %p", handle, (void *) fd);
	if (!m_bIsScanning)
	{
		XDBG(DEBUG_SANE, "%s","sane_get_select_fd: not scanning");
		return SANE_STATUS_INVAL;
	}
	return SANE_STATUS_UNSUPPORTED;
}
